@@ -46,11 +46,11 @@ use super::{adjusted_precision_scale, max_value, number_digits};
 /// use arrow2::array::PrimitiveArray;
 /// use arrow2::datatypes::DataType;
 ///
-/// let a = PrimitiveArray::from(&vec![Some(1_00i128), Some(4_00i128), Some(6_00i128)]).to(DataType::Decimal(5, 2));
-/// let b = PrimitiveArray::from(&vec![Some(1_00i128), Some(2_00i128), Some(2_00i128)]).to(DataType::Decimal(5, 2));
+/// let a = PrimitiveArray::from([Some(1_00i128), Some(4_00i128), Some(6_00i128)]).to(DataType::Decimal(5, 2));
+/// let b = PrimitiveArray::from([Some(1_00i128), Some(2_00i128), Some(2_00i128)]).to(DataType::Decimal(5, 2));
 ///
 /// let result = div(&a, &b).unwrap();
-/// let expected = PrimitiveArray::from(&vec![Some(1_00i128), Some(2_00i128), Some(3_00i128)]).to(DataType::Decimal(5, 2));
+/// let expected = PrimitiveArray::from([Some(1_00i128), Some(2_00i128), Some(3_00i128)]).to(DataType::Decimal(5, 2));
 ///
 /// assert_eq!(result, expected);
 /// ```
@@ -115,11 +115,11 @@ pub fn div(lhs: &PrimitiveArray<i128>, rhs: &PrimitiveArray<i128>) -> Result<Pri
 /// use arrow2::array::PrimitiveArray;
 /// use arrow2::datatypes::DataType;
 ///
-/// let a = PrimitiveArray::from(&vec![Some(999_99i128), Some(4_00i128), Some(6_00i128)]).to(DataType::Decimal(5, 2));
-/// let b = PrimitiveArray::from(&vec![Some(000_01i128), Some(2_00i128), Some(2_00i128)]).to(DataType::Decimal(5, 2));
+/// let a = PrimitiveArray::from([Some(999_99i128), Some(4_00i128), Some(6_00i128)]).to(DataType::Decimal(5, 2));
+/// let b = PrimitiveArray::from([Some(000_01i128), Some(2_00i128), Some(2_00i128)]).to(DataType::Decimal(5, 2));
 ///
 /// let result = saturating_div(&a, &b).unwrap();
-/// let expected = PrimitiveArray::from(&vec![Some(999_99i128), Some(2_00i128), Some(3_00i128)]).to(DataType::Decimal(5, 2));
+/// let expected = PrimitiveArray::from([Some(999_99i128), Some(2_00i128), Some(3_00i128)]).to(DataType::Decimal(5, 2));
 ///
 /// assert_eq!(result, expected);
 /// ```
@@ -180,11 +180,11 @@ pub fn saturating_div(
 /// use arrow2::array::PrimitiveArray;
 /// use arrow2::datatypes::DataType;
 ///
-/// let a = PrimitiveArray::from(&vec![Some(1_00i128), Some(4_00i128), Some(6_00i128)]).to(DataType::Decimal(5, 2));
-/// let b = PrimitiveArray::from(&vec![Some(000_00i128), None, Some(2_00i128)]).to(DataType::Decimal(5, 2));
+/// let a = PrimitiveArray::from([Some(1_00i128), Some(4_00i128), Some(6_00i128)]).to(DataType::Decimal(5, 2));
+/// let b = PrimitiveArray::from([Some(000_00i128), None, Some(2_00i128)]).to(DataType::Decimal(5, 2));
 ///
 /// let result = checked_div(&a, &b).unwrap();
-/// let expected = PrimitiveArray::from(&vec![None, None, Some(3_00i128)]).to(DataType::Decimal(5, 2));
+/// let expected = PrimitiveArray::from([None, None, Some(3_00i128)]).to(DataType::Decimal(5, 2));
 ///
 /// assert_eq!(result, expected);
 /// ```
@@ -261,10 +261,10 @@ impl ArrayCheckedDiv<PrimitiveArray<i128>> for PrimitiveArray<i128> {
 /// use arrow2::array::PrimitiveArray;
 /// use arrow2::datatypes::DataType;
 ///
-/// let a = PrimitiveArray::from(&vec![Some(1000_00i128)]).to(DataType::Decimal(7, 2));
-/// let b = PrimitiveArray::from(&vec![Some(10_0000i128)]).to(DataType::Decimal(6, 4));
+/// let a = PrimitiveArray::from([Some(1000_00i128)]).to(DataType::Decimal(7, 2));
+/// let b = PrimitiveArray::from([Some(10_0000i128)]).to(DataType::Decimal(6, 4));
 /// let result = adaptive_div(&a, &b).unwrap();
-/// let expected = PrimitiveArray::from(&vec![Some(100_0000i128)]).to(DataType::Decimal(9, 4));
+/// let expected = PrimitiveArray::from([Some(100_0000i128)]).to(DataType::Decimal(9, 4));
 ///
 /// assert_eq!(result, expected);
 /// ```
@@ -345,7 +345,7 @@ mod tests {
         //   123.456 -->     123456
         // --------       ---------
         //     1.800 <--       1800
-        let a = PrimitiveArray::from(&vec![
+        let a = PrimitiveArray::from([
             Some(222_222i128),
             Some(10_000i128),
             Some(20_000i128),
@@ -355,7 +355,7 @@ mod tests {
         ])
         .to(DataType::Decimal(7, 3));
 
-        let b = PrimitiveArray::from(&vec![
+        let b = PrimitiveArray::from([
             Some(123_456i128),
             Some(2_000i128),
             Some(3_000i128),
@@ -366,7 +366,7 @@ mod tests {
         .to(DataType::Decimal(7, 3));
 
         let result = div(&a, &b).unwrap();
-        let expected = PrimitiveArray::from(&vec![
+        let expected = PrimitiveArray::from([
             Some(1_800i128),
             Some(5_000i128),
             Some(6_666i128),
@@ -385,8 +385,8 @@ mod tests {
 
     #[test]
     fn test_divide_decimal_wrong_precision() {
-        let a = PrimitiveArray::from(&vec![None]).to(DataType::Decimal(5, 2));
-        let b = PrimitiveArray::from(&vec![None]).to(DataType::Decimal(6, 2));
+        let a = PrimitiveArray::from([None]).to(DataType::Decimal(5, 2));
+        let b = PrimitiveArray::from([None]).to(DataType::Decimal(6, 2));
         let result = div(&a, &b);
 
         if result.is_ok() {
@@ -397,14 +397,14 @@ mod tests {
     #[test]
     #[should_panic(expected = "Overflow in multiplication presented for precision 5")]
     fn test_divide_panic() {
-        let a = PrimitiveArray::from(&vec![Some(99999i128)]).to(DataType::Decimal(5, 2));
-        let b = PrimitiveArray::from(&vec![Some(000_01i128)]).to(DataType::Decimal(5, 2));
+        let a = PrimitiveArray::from([Some(99999i128)]).to(DataType::Decimal(5, 2));
+        let b = PrimitiveArray::from([Some(000_01i128)]).to(DataType::Decimal(5, 2));
         let _ = div(&a, &b);
     }
 
     #[test]
     fn test_divide_saturating() {
-        let a = PrimitiveArray::from(&vec![
+        let a = PrimitiveArray::from([
             Some(222_222i128),
             Some(10_000i128),
             Some(20_000i128),
@@ -414,7 +414,7 @@ mod tests {
         ])
         .to(DataType::Decimal(7, 3));
 
-        let b = PrimitiveArray::from(&vec![
+        let b = PrimitiveArray::from([
             Some(123_456i128),
             Some(2_000i128),
             Some(3_000i128),
@@ -425,7 +425,7 @@ mod tests {
         .to(DataType::Decimal(7, 3));
 
         let result = saturating_div(&a, &b).unwrap();
-        let expected = PrimitiveArray::from(&vec![
+        let expected = PrimitiveArray::from([
             Some(1_800i128),
             Some(5_000i128),
             Some(6_666i128),
@@ -440,7 +440,7 @@ mod tests {
 
     #[test]
     fn test_divide_saturating_overflow() {
-        let a = PrimitiveArray::from(&vec![
+        let a = PrimitiveArray::from([
             Some(99999i128),
             Some(99999i128),
             Some(99999i128),
@@ -448,7 +448,7 @@ mod tests {
             Some(99999i128),
         ])
         .to(DataType::Decimal(5, 2));
-        let b = PrimitiveArray::from(&vec![
+        let b = PrimitiveArray::from([
             Some(-00001i128),
             Some(00001i128),
             Some(00010i128),
@@ -459,7 +459,7 @@ mod tests {
 
         let result = saturating_div(&a, &b).unwrap();
 
-        let expected = PrimitiveArray::from(&vec![
+        let expected = PrimitiveArray::from([
             Some(-99999i128),
             Some(99999i128),
             Some(99999i128),
@@ -473,7 +473,7 @@ mod tests {
 
     #[test]
     fn test_divide_checked() {
-        let a = PrimitiveArray::from(&vec![
+        let a = PrimitiveArray::from([
             Some(222_222i128),
             Some(10_000i128),
             Some(20_000i128),
@@ -483,7 +483,7 @@ mod tests {
         ])
         .to(DataType::Decimal(7, 3));
 
-        let b = PrimitiveArray::from(&vec![
+        let b = PrimitiveArray::from([
             Some(123_456i128),
             Some(2_000i128),
             Some(3_000i128),
@@ -494,7 +494,7 @@ mod tests {
         .to(DataType::Decimal(7, 3));
 
         let result = div(&a, &b).unwrap();
-        let expected = PrimitiveArray::from(&vec![
+        let expected = PrimitiveArray::from([
             Some(1_800i128),
             Some(5_000i128),
             Some(6_666i128),
@@ -509,14 +509,14 @@ mod tests {
 
     #[test]
     fn test_divide_checked_overflow() {
-        let a = PrimitiveArray::from(&vec![Some(1_00i128), Some(4_00i128), Some(6_00i128)])
+        let a = PrimitiveArray::from([Some(1_00i128), Some(4_00i128), Some(6_00i128)])
             .to(DataType::Decimal(5, 2));
-        let b = PrimitiveArray::from(&vec![Some(000_00i128), None, Some(2_00i128)])
+        let b = PrimitiveArray::from([Some(000_00i128), None, Some(2_00i128)])
             .to(DataType::Decimal(5, 2));
 
         let result = checked_div(&a, &b).unwrap();
         let expected =
-            PrimitiveArray::from(&vec![None, None, Some(3_00i128)]).to(DataType::Decimal(5, 2));
+            PrimitiveArray::from([None, None, Some(3_00i128)]).to(DataType::Decimal(5, 2));
 
         assert_eq!(result, expected);
 
@@ -531,11 +531,11 @@ mod tests {
         //    10.0000 -> 6, 4
         // -----------------
         //   100.0000 -> 9, 4
-        let a = PrimitiveArray::from(&vec![Some(1000_00i128)]).to(DataType::Decimal(7, 2));
-        let b = PrimitiveArray::from(&vec![Some(10_0000i128)]).to(DataType::Decimal(6, 4));
+        let a = PrimitiveArray::from([Some(1000_00i128)]).to(DataType::Decimal(7, 2));
+        let b = PrimitiveArray::from([Some(10_0000i128)]).to(DataType::Decimal(6, 4));
         let result = adaptive_div(&a, &b).unwrap();
 
-        let expected = PrimitiveArray::from(&vec![Some(100_0000i128)]).to(DataType::Decimal(9, 4));
+        let expected = PrimitiveArray::from([Some(100_0000i128)]).to(DataType::Decimal(9, 4));
 
         assert_eq!(result, expected);
         assert_eq!(result.data_type(), &DataType::Decimal(9, 4));
@@ -544,11 +544,11 @@ mod tests {
         //      10.002  -> 5, 3
         // -----------------
         //    1110.877  -> 8, 3
-        let a = PrimitiveArray::from(&vec![Some(11111_0i128)]).to(DataType::Decimal(6, 1));
-        let b = PrimitiveArray::from(&vec![Some(10_002i128)]).to(DataType::Decimal(5, 3));
+        let a = PrimitiveArray::from([Some(11111_0i128)]).to(DataType::Decimal(6, 1));
+        let b = PrimitiveArray::from([Some(10_002i128)]).to(DataType::Decimal(5, 3));
         let result = adaptive_div(&a, &b).unwrap();
 
-        let expected = PrimitiveArray::from(&vec![Some(1110_877i128)]).to(DataType::Decimal(8, 3));
+        let expected = PrimitiveArray::from([Some(1110_877i128)]).to(DataType::Decimal(8, 3));
 
         assert_eq!(result, expected);
         assert_eq!(result.data_type(), &DataType::Decimal(8, 3));
@@ -557,11 +557,11 @@ mod tests {
         //     12345.678  ->  8, 3
         // -----------------
         //         0.999  ->  8, 3
-        let a = PrimitiveArray::from(&vec![Some(12345_67i128)]).to(DataType::Decimal(7, 2));
-        let b = PrimitiveArray::from(&vec![Some(12345_678i128)]).to(DataType::Decimal(8, 3));
+        let a = PrimitiveArray::from([Some(12345_67i128)]).to(DataType::Decimal(7, 2));
+        let b = PrimitiveArray::from([Some(12345_678i128)]).to(DataType::Decimal(8, 3));
         let result = adaptive_div(&a, &b).unwrap();
 
-        let expected = PrimitiveArray::from(&vec![Some(0_999i128)]).to(DataType::Decimal(8, 3));
+        let expected = PrimitiveArray::from([Some(0_999i128)]).to(DataType::Decimal(8, 3));
 
         assert_eq!(result, expected);
         assert_eq!(result.data_type(), &DataType::Decimal(8, 3));
