@@ -19,9 +19,9 @@
 
 use crate::{
     array::{new_empty_array, Array, NullArray, PrimitiveArray},
-    datatypes::{DataType, IntervalUnit},
+    datatypes::DataType,
     error::Result,
-    types::{days_ms, Index},
+    types::Index,
 };
 
 mod binary;
@@ -33,87 +33,56 @@ mod primitive;
 mod structure;
 mod utf8;
 
-macro_rules! downcast_take {
-    ($type: ty, $values: expr, $indices: expr) => {{
-        let values = $values
-            .as_any()
-            .downcast_ref()
-            .expect("Unable to downcast to a primitive array");
-        Ok(Box::new(primitive::take::<$type, _>(&values, $indices)))
-    }};
-}
-
-macro_rules! downcast_dict_take {
-    ($type: ty, $values: expr, $indices: expr) => {{
-        let values = $values
-            .as_any()
-            .downcast_ref()
-            .expect("Unable to downcast to a primitive array");
-        Ok(Box::new(dict::take::<$type, _>(&values, $indices)))
-    }};
-}
-
 pub fn take<O: Index>(values: &dyn Array, indices: &PrimitiveArray<O>) -> Result<Box<dyn Array>> {
     if indices.len() == 0 {
         return Ok(new_empty_array(values.data_type().clone()));
     }
 
-    match values.data_type() {
-        DataType::Null => Ok(Box::new(NullArray::from_data(indices.len()))),
-        DataType::Boolean => {
+    use crate::datatypes::PhysicalType::*;
+    match values.data_type().to_physical_type() {
+        Null => Ok(Box::new(NullArray::from_data(
+            values.data_type().clone(),
+            indices.len(),
+        ))),
+        Boolean => {
             let values = values.as_any().downcast_ref().unwrap();
             Ok(Box::new(boolean::take::<O>(values, indices)))
         }
-        DataType::Int8 => downcast_take!(i8, values, indices),
-        DataType::Int16 => downcast_take!(i16, values, indices),
-        DataType::Int32
-        | DataType::Date32
-        | DataType::Time32(_)
-        | DataType::Interval(IntervalUnit::YearMonth) => downcast_take!(i32, values, indices),
-        DataType::Int64
-        | DataType::Date64
-        | DataType::Time64(_)
-        | DataType::Duration(_)
-        | DataType::Timestamp(_, _) => downcast_take!(i64, values, indices),
-        DataType::Interval(IntervalUnit::DayTime) => downcast_take!(days_ms, values, indices),
-        DataType::UInt8 => downcast_take!(u8, values, indices),
-        DataType::UInt16 => downcast_take!(u16, values, indices),
-        DataType::UInt32 => downcast_take!(u32, values, indices),
-        DataType::UInt64 => downcast_take!(u64, values, indices),
-        DataType::Float16 => unreachable!(),
-        DataType::Float32 => downcast_take!(f32, values, indices),
-        DataType::Float64 => downcast_take!(f64, values, indices),
-        DataType::Decimal(_, _) => downcast_take!(i128, values, indices),
-        DataType::Utf8 => {
+        Primitive(primitive) => with_match_primitive_type!(primitive, |$T| {
+            let values = values.as_any().downcast_ref().unwrap();
+            Ok(Box::new(primitive::take::<$T, _>(&values, indices)))
+        }),
+        Utf8 => {
             let values = values.as_any().downcast_ref().unwrap();
             Ok(Box::new(utf8::take::<i32, _>(values, indices)))
         }
-        DataType::LargeUtf8 => {
+        LargeUtf8 => {
             let values = values.as_any().downcast_ref().unwrap();
             Ok(Box::new(utf8::take::<i64, _>(values, indices)))
         }
-        DataType::Binary => {
+        Binary => {
             let values = values.as_any().downcast_ref().unwrap();
             Ok(Box::new(binary::take::<i32, _>(values, indices)))
         }
-        DataType::LargeBinary => {
+        LargeBinary => {
             let values = values.as_any().downcast_ref().unwrap();
             Ok(Box::new(binary::take::<i64, _>(values, indices)))
         }
-        DataType::Dictionary(key_type, _) => {
-            with_match_dictionary_key_type!(key_type.as_ref(), |$T| {
-                downcast_dict_take!($T, values, indices)
+        Dictionary(key_type) => {
+            with_match_physical_dictionary_key_type!(key_type, |$T| {
+                let values = values.as_any().downcast_ref().unwrap();
+                Ok(Box::new(dict::take::<$T, _>(&values, indices)))
             })
         }
-        DataType::Struct(_) => {
+        Struct => {
             let array = values.as_any().downcast_ref().unwrap();
             Ok(Box::new(structure::take::<_>(array, indices)?))
         }
-        DataType::List(_) => {
+        List => {
             let array = values.as_any().downcast_ref().unwrap();
             Ok(Box::new(list::take::<i32, O>(array, indices)))
         }
-        DataType::LargeList(_) => {
+        LargeList => {
             let array = values.as_any().downcast_ref().unwrap();
             Ok(Box::new(list::take::<i64, O>(array, indices)))
         }
@@ -180,7 +149,7 @@ pub fn can_take(data_type: &DataType) -> bool {
 mod tests {
     use std::sync::Arc;
 
-    use crate::datatypes::Field;
+    use crate::datatypes::{Field, IntervalUnit};
     use crate::{array::*, bitmap::MutableBitmap, types::NativeType};
 
     use super::*;
@@ -253,7 +222,7 @@ mod tests {
             Field::new("b", DataType::Int32, true),
         ];
         StructArray::from_data(
-            fields,
+            DataType::Struct(fields),
             vec![
                 Arc::new(boolean) as Arc<dyn Array>,
                 Arc::new(int) as Arc<dyn Array>,
@@ -277,7 +246,7 @@ mod tests {
             .collect::<MutableBitmap>()
             .into();
         let expected = StructArray::from_data(
-            array.fields().to_vec(),
+            array.data_type().clone(),
             vec![
                 Arc::new(boolean) as Arc<dyn Array>,
                 Arc::new(int) as Arc<dyn Array>,
