@@ -1,12 +1,15 @@
-use std::{ffi::CStr, ffi::CString, ptr};
+use std::{collections::BTreeMap, convert::TryInto, ffi::CStr, ffi::CString, ptr};
 
 use crate::{
-    datatypes::{DataType, Field, IntervalUnit, TimeUnit},
+    datatypes::{DataType, Extension, Field, IntervalUnit, Metadata, TimeUnit},
     error::{ArrowError, Result},
 };
 
 #[allow(dead_code)]
 struct SchemaPrivateData {
+    name: CString,
+    format: CString,
+    metadata: Option<Vec<u8>>,
     children_ptr: Box<[*mut Ffi_ArrowSchema]>,
     dictionary: Option<*mut Ffi_ArrowSchema>,
 }
@@ -35,9 +38,6 @@ unsafe extern "C" fn c_release_schema(schema: *mut Ffi_ArrowSchema) {
     }
     let schema = &mut *schema;
 
-    // take ownership back to release it.
-    CString::from_raw(schema.format as *mut std::os::raw::c_char);
-    CString::from_raw(schema.name as *mut std::os::raw::c_char);
     let private = Box::from_raw(schema.private_data as *mut SchemaPrivateData);
     for child in private.children_ptr.iter() {
         let _ = Box::from_raw(*child);
@@ -91,16 +91,47 @@ impl Ffi_ArrowSchema {
             None
         };
 
+        let metadata = field.metadata();
+
+        let metadata = if let DataType::Extension(name, _, extension_metadata) = field.data_type() {
+            // append extension information.
+            let mut metadata = metadata.clone().unwrap_or_default();
+
+            // metadata
+            if let Some(extension_metadata) = extension_metadata {
+                metadata.insert(
+                    "ARROW:extension:metadata".to_string(),
+                    extension_metadata.clone(),
+                );
+            }
+
+            metadata.insert("ARROW:extension:name".to_string(), name.clone());
+
+            Some(metadata_to_bytes(&metadata))
+        } else {
+            metadata.as_ref().map(metadata_to_bytes)
+        };
+
+        let name = CString::new(name).unwrap();
+        let format = CString::new(format).unwrap();
+
         let mut private = Box::new(SchemaPrivateData {
+            name,
+            format,
+            metadata,
             children_ptr,
             dictionary: dictionary.map(Box::into_raw),
         });
 
         // <https://arrow.apache.org/docs/format/CDataInterface.html#c.ArrowSchema>
         Self {
-            format: CString::new(format).unwrap().into_raw(),
-            name: CString::new(name).unwrap().into_raw(),
-            metadata: std::ptr::null_mut(),
+            format: private.format.as_ptr(),
+            name: private.name.as_ptr(),
+            metadata: private
+                .metadata
+                .as_ref()
+                .map(|x| x.as_ptr())
+                .unwrap_or(std::ptr::null()) as *const ::std::os::raw::c_char,
             flags,
             n_children,
             children: private.children_ptr.as_mut_ptr(),
@@ -180,7 +211,17 @@ pub fn to_field(schema: &Ffi_ArrowSchema) -> Result<Field> {
     } else {
         to_data_type(schema)?
     };
-    Ok(Field::new(schema.name(), data_type, schema.nullable()))
+    let (metadata, extension) = unsafe { metadata_from_bytes(schema.metadata) };
+
+    let data_type = if let Some((name, extension_metadata)) = extension {
+        DataType::Extension(name, Box::new(data_type), extension_metadata)
+    } else {
+        data_type
+    };
+
+    let mut field = Field::new(schema.name(), data_type, schema.nullable());
+    field.set_metadata(metadata);
+    Ok(field)
 }
 
 fn to_data_type(schema: &Ffi_ArrowSchema) -> Result<DataType> {
@@ -323,6 +364,9 @@ fn to_format(data_type: &DataType) -> String {
         DataType::Duration(TimeUnit::Nanosecond) => "tDn".to_string(),
         DataType::Interval(IntervalUnit::YearMonth) => "tiM".to_string(),
         DataType::Interval(IntervalUnit::DayTime) => "tiD".to_string(),
+        DataType::Interval(IntervalUnit::MonthDayNano) => {
+            todo!("Spec for FFI for MonthDayNano still not defined.")
+        }
         DataType::Timestamp(unit, tz) => {
             let unit = match unit {
                 TimeUnit::Second => "s".to_string(),
@@ -356,6 +400,7 @@ fn to_format(data_type: &DataType) -> String {
             r
         }
         DataType::Dictionary(index, _) => to_format(index.as_ref()),
+        DataType::Extension(_, inner, _) => to_format(inner.as_ref()),
     }
 }
 
@@ -371,3 +416,60 @@ pub(super) fn get_field_child(field: &Field, index: usize) -> Result<Field> {
         ))),
     }
 }
+
+fn metadata_to_bytes(metadata: &BTreeMap<String, String>) -> Vec<u8> {
+    let a = (metadata.len() as i32).to_ne_bytes().to_vec();
+    metadata.iter().fold(a, |mut acc, (key, value)| {
+        acc.extend((key.len() as i32).to_ne_bytes());
+        acc.extend(key.as_bytes());
+        acc.extend((value.len() as i32).to_ne_bytes());
+        acc.extend(value.as_bytes());
+        acc
+    })
+}
+
+unsafe fn read_ne_i32(ptr: *const u8) -> i32 {
+    let slice = std::slice::from_raw_parts(ptr, 4);
+    i32::from_ne_bytes(slice.try_into().unwrap())
+}
+
+unsafe fn read_bytes(ptr: *const u8, len: usize) -> &'static str {
+    let slice = std::slice::from_raw_parts(ptr, len);
+    std::str::from_utf8(slice).unwrap()
+}
+
+unsafe fn metadata_from_bytes(data: *const ::std::os::raw::c_char) -> (Metadata, Extension) {
+    let mut data = data as *const u8; // u8 = i8
+    if data.is_null() {
+        return (None, None);
+    };
+    let len = read_ne_i32(data);
+    data = data.add(4);
+
+    let mut result = BTreeMap::new();
+    let mut extension_name = None;
+    let mut extension_metadata = None;
+    for _ in 0..len {
+        let key_len = read_ne_i32(data) as usize;
+        data = data.add(4);
+        let key = read_bytes(data, key_len);
+        data = data.add(key_len);
+        let value_len = read_ne_i32(data) as usize;
+        data = data.add(4);
+        let value = read_bytes(data, value_len);
+        data = data.add(value_len);
+        match key {
+            "ARROW:extension:name" => {
+                extension_name = Some(value.to_string());
+            }
+            "ARROW:extension:metadata" => {
+                extension_metadata = Some(value.to_string());
+            }
+            _ => {
+                result.insert(key.to_string(), value.to_string());
+            }
+        };
+    }
+    let extension = extension_name.map(|name| (name, extension_metadata));
+    (Some(result), extension)
+}
