@@ -1,13 +1,13 @@
 use std::sync::Arc;
 
 use parquet2::{
-    encoding::{bitpacking, hybrid_rle, uleb128, Encoding},
+    encoding::{hybrid_rle, Encoding},
     page::{DataPage, PrimitivePageDict},
     read::StreamingIterator,
     types::NativeType,
 };
 
-use super::super::utils as other_utils;
+use super::super::utils;
 use super::{ColumnChunkMetaData, ColumnDescriptor};
 use crate::{
     array::{Array, DictionaryArray, DictionaryKey, PrimitiveArray},
@@ -18,6 +18,7 @@ use crate::{
     types::NativeType as ArrowNativeType,
 };
 
+#[allow(clippy::too_many_arguments)]
 fn read_dict_optional<K, T, A, F>(
     validity_buffer: &[u8],
     indices_buffer: &[u8],
@@ -41,12 +42,9 @@ fn read_dict_optional<K, T, A, F>(
     let bit_width = indices_buffer[0];
     let indices_buffer = &indices_buffer[1..];
 
-    let (_, consumed) = uleb128::decode(indices_buffer);
-    let indices_buffer = &indices_buffer[consumed..];
-
-    let non_null_indices_len = indices_buffer.len() * 8 / bit_width as usize;
-
-    let mut new_indices = bitpacking::Decoder::new(indices_buffer, bit_width, non_null_indices_len);
+    println!("indices_buffer: {:?}", indices_buffer);
+    let mut new_indices =
+        hybrid_rle::HybridRleDecoder::new(indices_buffer, bit_width as u32, additional);
 
     let validity_iterator = hybrid_rle::Decoder::new(validity_buffer, 1);
 
@@ -100,7 +98,7 @@ where
     assert_eq!(descriptor.max_rep_level(), 0);
     let is_optional = descriptor.max_def_level() == 1;
 
-    let (validity_buffer, values_buffer, version) = other_utils::split_buffer(page, is_optional);
+    let (_, validity_buffer, values_buffer, version) = utils::split_buffer(page, descriptor);
 
     match (&page.encoding(), page.dictionary_page(), is_optional) {
         (Encoding::PlainDictionary | Encoding::RleDictionary, Some(dict), true) => {
@@ -116,7 +114,7 @@ where
             )
         }
         _ => {
-            return Err(other_utils::not_implemented(
+            return Err(utils::not_implemented(
                 &page.encoding(),
                 is_optional,
                 page.dictionary_page().is_some(),
