@@ -1,4 +1,4 @@
-//! This module contains traits to handle all _physical_ types used in this crate.
+//! traits to handle _all physical types_ used in this crate.
 //! Most physical types used in this crate are native Rust types, like `i32`.
 //! The most important trait is [`NativeType`], implemented for all Arrow types
 //! with a Rust correspondence (such as `i32` or `f64`).
@@ -15,24 +15,27 @@ mod index;
 pub mod simd;
 pub use index::*;
 
-use crate::datatypes::{DataType, IntervalUnit, TimeUnit};
+use crate::datatypes::{DataType, IntervalUnit, PhysicalType, PrimitiveType};
 
 /// Trait denoting anything that has a natural logical [`DataType`].
 /// For example, [`DataType::Int32`] for `i32`.
 pub trait NaturalDataType {
+    /// The natural [`DataType`].
     const DATA_TYPE: DataType;
 }
 
+/// describes whether a [`DataType`] is valid.
 pub unsafe trait Relation {
+    /// Whether `data_type` is a valid [`DataType`].
     fn is_valid(data_type: &DataType) -> bool;
 }
 
 macro_rules! create_relation {
-    ($native_ty:ty, $($impl_pattern:pat)|+) => {
+    ($native_ty:ty, $physical_ty:expr) => {
         unsafe impl Relation for $native_ty {
             #[inline]
             fn is_valid(data_type: &DataType) -> bool {
-                matches!(data_type, $($impl_pattern)|+)
+                data_type.to_physical_type() == $physical_ty
             }
         }
     };
@@ -46,7 +49,7 @@ macro_rules! natural_type {
     };
 }
 
-/// Trait declaring any type that can be allocated, serialized and deserialized by this crate.
+/// Declares any type that can be allocated, serialized and deserialized by this crate.
 /// All data-heavy memory operations are implemented for this trait alone.
 /// # Safety
 /// Do not implement.
@@ -64,12 +67,16 @@ pub unsafe trait NativeType:
     + Sized
     + 'static
 {
+    /// Type denoting its representation as bytes
     type Bytes: AsRef<[u8]> + for<'a> TryFrom<&'a [u8]>;
 
+    /// To bytes in little endian
     fn to_le_bytes(&self) -> Self::Bytes;
 
+    /// To bytes in big endian
     fn to_be_bytes(&self) -> Self::Bytes;
 
+    /// From bytes in big endian
     fn from_be_bytes(bytes: Self::Bytes) -> Self;
 }
 
@@ -118,36 +125,23 @@ natural_type!(i64, DataType::Int64);
 natural_type!(f32, DataType::Float32);
 natural_type!(f64, DataType::Float64);
 natural_type!(days_ms, DataType::Interval(IntervalUnit::DayTime));
+natural_type!(
+    months_days_ns,
+    DataType::Interval(IntervalUnit::MonthDayNano)
+);
 natural_type!(i128, DataType::Decimal(32, 32)); // users should set the decimal when creating an array
 
-create_relation!(u8, &DataType::UInt8);
-create_relation!(u16, &DataType::UInt16);
-create_relation!(u32, &DataType::UInt32);
-create_relation!(u64, &DataType::UInt64);
-create_relation!(i8, &DataType::Int8);
-create_relation!(i16, &DataType::Int16);
-create_relation!(
-    i32,
-    &DataType::Int32
-        | &DataType::Date32
-        | &DataType::Time32(TimeUnit::Millisecond)
-        | &DataType::Time32(TimeUnit::Second)
-        | &DataType::Interval(IntervalUnit::YearMonth)
-);
-
-create_relation!(
-    i64,
-    &DataType::Int64
-        | &DataType::Date64
-        | &DataType::Time64(TimeUnit::Microsecond)
-        | &DataType::Time64(TimeUnit::Nanosecond)
-        | &DataType::Timestamp(_, _)
-        | &DataType::Duration(_)
-);
-
-create_relation!(i128, &DataType::Decimal(_, _));
-create_relation!(f32, &DataType::Float32);
-create_relation!(f64, &DataType::Float64);
+create_relation!(u8, PhysicalType::Primitive(PrimitiveType::UInt8));
+create_relation!(u16, PhysicalType::Primitive(PrimitiveType::UInt16));
+create_relation!(u32, PhysicalType::Primitive(PrimitiveType::UInt32));
+create_relation!(u64, PhysicalType::Primitive(PrimitiveType::UInt64));
+create_relation!(i8, PhysicalType::Primitive(PrimitiveType::Int8));
+create_relation!(i16, PhysicalType::Primitive(PrimitiveType::Int16));
+create_relation!(i32, PhysicalType::Primitive(PrimitiveType::Int32));
+create_relation!(i64, PhysicalType::Primitive(PrimitiveType::Int64));
+create_relation!(i128, PhysicalType::Primitive(PrimitiveType::Int128));
+create_relation!(f32, PhysicalType::Primitive(PrimitiveType::Float32));
+create_relation!(f64, PhysicalType::Primitive(PrimitiveType::Float64));
 
 /// The in-memory representation of the DayMillisecond variant of arrow's "Interval" logical type.
 #[derive(Debug, Copy, Clone, Default, PartialEq, Eq, Hash)]
@@ -210,21 +204,129 @@ unsafe impl NativeType for days_ms {
     }
 }
 
-create_relation!(days_ms, &DataType::Interval(IntervalUnit::DayTime));
+create_relation!(days_ms, PhysicalType::Primitive(PrimitiveType::DaysMs));
 
 impl days_ms {
+    /// A new [`days_ms`].
     #[inline]
     pub fn new(days: i32, milliseconds: i32) -> Self {
         Self([days, milliseconds])
     }
 
+    /// The number of days
     #[inline]
     pub fn days(&self) -> i32 {
         self.0[0]
     }
 
+    /// The number of milliseconds
     #[inline]
     pub fn milliseconds(&self) -> i32 {
         self.0[1]
     }
 }
+
+/// The in-memory representation of the MonthDayNano variant of the "Interval" logical type.
+#[derive(Debug, Copy, Clone, Default, PartialEq, Eq, Hash)]
+#[allow(non_camel_case_types)]
+#[repr(C)]
+pub struct months_days_ns(i32, i32, i64);
+
+impl std::fmt::Display for months_days_ns {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "{}m {}d {}ns", self.months(), self.days(), self.ns())
+    }
+}
+
+unsafe impl NativeType for months_days_ns {
+    type Bytes = [u8; 16];
+    #[inline]
+    fn to_le_bytes(&self) -> Self::Bytes {
+        let months = self.months().to_le_bytes();
+        let days = self.days().to_le_bytes();
+        let ns = self.ns().to_le_bytes();
+        let mut result = [0; 16];
+        result[0] = months[0];
+        result[1] = months[1];
+        result[2] = months[2];
+        result[3] = months[3];
+        result[4] = days[0];
+        result[5] = days[1];
+        result[6] = days[2];
+        result[7] = days[3];
+        (0..8).for_each(|i| {
+            result[8 + i] = ns[i];
+        });
+        result
+    }
+
+    #[inline]
+    fn to_be_bytes(&self) -> Self::Bytes {
+        let months = self.months().to_be_bytes();
+        let days = self.days().to_be_bytes();
+        let ns = self.ns().to_be_bytes();
+        let mut result = [0; 16];
+        result[0] = months[0];
+        result[1] = months[1];
+        result[2] = months[2];
+        result[3] = months[3];
+        result[4] = days[0];
+        result[5] = days[1];
+        result[6] = days[2];
+        result[7] = days[3];
+        (0..8).for_each(|i| {
+            result[8 + i] = ns[i];
+        });
+        result
+    }
+
+    #[inline]
+    fn from_be_bytes(bytes: Self::Bytes) -> Self {
+        let mut months = [0; 4];
+        months[0] = bytes[0];
+        months[1] = bytes[1];
+        months[2] = bytes[2];
+        months[3] = bytes[3];
+        let mut days = [0; 4];
+        days[0] = bytes[4];
+        days[1] = bytes[5];
+        days[2] = bytes[6];
+        days[3] = bytes[7];
+        let mut ns = [0; 8];
+        (0..8).for_each(|i| {
+            ns[i] = bytes[8 + i];
+        });
+        Self(
+            i32::from_be_bytes(months),
+            i32::from_be_bytes(days),
+            i64::from_be_bytes(ns),
+        )
+    }
+}
+
+create_relation!(
+    months_days_ns,
+    PhysicalType::Primitive(PrimitiveType::MonthDayNano)
+);
+
+impl months_days_ns {
+    #[inline]
+    pub fn new(months: i32, days: i32, nanoseconds: i64) -> Self {
+        Self(months, days, nanoseconds)
+    }
+
+    #[inline]
+    pub fn months(&self) -> i32 {
+        self.0
+    }
+
+    #[inline]
+    pub fn days(&self) -> i32 {
+        self.1
+    }
+
+    #[inline]
+    pub fn ns(&self) -> i64 {
+        self.2
+    }
+}
