@@ -1,6 +1,13 @@
-use arrow2::{array::*, bitmap::Bitmap, datatypes::*, types::days_ms};
 use std::iter::FromIterator;
 
+use arrow2::{
+    array::*,
+    bitmap::Bitmap,
+    buffer::Buffer,
+    datatypes::*,
+    types::{days_ms, months_days_ns},
+};
+
 mod mutable;
 
 #[test]
@@ -175,6 +182,18 @@ fn display_timestamp_ns() {
     );
 }
 
+#[test]
+fn display_timestamp_tz_ns() {
+    let array = Int64Array::from(&[Some(1), None, Some(2)]).to(DataType::Timestamp(
+        TimeUnit::Nanosecond,
+        Some("+02:00".to_string()),
+    ));
+    assert_eq!(
+        format!("{}", array),
+        "Timestamp(Nanosecond, Some(\"+02:00\"))[1970-01-01 02:00:00.000000001 +02:00, , 1970-01-01 02:00:00.000000002 +02:00]"
+    );
+}
+
 #[test]
 fn display_duration_ms() {
     let array =
@@ -220,3 +239,40 @@ fn display_interval_days_ms() {
     let array = DaysMsArray::from(&[Some(days_ms::new(1, 1)), None, Some(days_ms::new(2, 2))]);
     assert_eq!(format!("{}", array), "Interval(DayTime)[1d1ms, , 2d2ms]");
 }
+
+#[test]
+fn display_months_days_ns() {
+    let data = &[
+        Some(months_days_ns::new(1, 1, 2)),
+        None,
+        Some(months_days_ns::new(2, 3, 3)),
+    ];
+
+    let array = MonthsDaysNsArray::from(&data);
+
+    assert_eq!(
+        format!("{}", array),
+        "Interval(MonthDayNano)[1m1d2ns, , 2m3d3ns]"
+    );
+}
+
+#[test]
+fn months_days_ns() {
+    let data = &[
+        months_days_ns::new(1, 1, 2),
+        months_days_ns::new(1, 1, 3),
+        months_days_ns::new(2, 3, 3),
+    ];
+
+    let array = MonthsDaysNsArray::from_slice(&data);
+
+    let a = array.values().as_slice();
+    assert_eq!(a, data.as_ref());
+}
+
+#[test]
+#[should_panic]
+fn wrong_data_type() {
+    let values = Buffer::from(b"abbb");
+    PrimitiveArray::from_data(DataType::Utf8, values, None);
+}
